/*
        cstrstream.h -- fast strstream operations
        (C) Copyright 1995  John Webster Small
        All rights reserved

        Dynamic ostrstream's and strstream's can be
        extremely slow when streaming.  The idea here
        is to allocate the largest string thought
        possibly to occur and then pack it once
        streaming operations have been completed.

        The CSTRSTRM_EXTRA_DEFAULT macro is used to
        specify the length of the largest string as
        a default parameter to ocstrstream() and
        cstrstream() constructors.  If 4095 is not
        suitable define CSTRSTRM_EXTRA_DEFAULT
        appropriately.  The reason for choosing 4095
        is so that limit() will be 4096.

        Don't forget to stream the "ends" manipulator
        before calling str()!  Only if pack() hasn't
        been called, may reset() be called.  However,
        pack() must be called to validate cstr len()
        and any subsequent cstr operations!
*/

#ifndef cstrstrm_h
#define cstrstrm_h

/* LINTLIBRARY */

// 0 < CSTRSTRM_EXTRA_DEFAULT < INT_MAX
#ifndef CSTRSTRM_EXTRA_DEFAULT
  #define CSTRSTRM_EXTRA_DEFAULT  4095
#endif

#ifndef cstr_h
  #include "cstr.h"
#endif

#ifndef strstream_h
  #include <strstream.h>
  #ifndef strstream_h
    #define strstream_h
  #endif
#endif

class orcstrstream : public ostrstream
{
    int _packed, _pcount, _packOnDestruct;
    cstr& cs_;
    size_t csStartLen;
  public:
    orcstrstream(cstr& cs, int packOnDestruct = 1);
    char * str() { return (char *)cs_.str(); }
    int pcount();
    strstreambuf * rdbuf() const
        { assert(0); return 0; }
    void reset();
    void pack();
    ~orcstrstream();
};

template <class TYPE>
inline cstr& operator<<(cstr & cs, const TYPE & t)
{ orcstrstream(cs,0) << t; return cs; }

template <class TYPE>
inline void operator>>(cstr & cs, TYPE & t)
{ istrstream((char *)cs.str()) >> t; }

class ocstrstream : cstr, public orcstrstream
{
  public:
    ocstrstream(int extra = CSTRSTRM_EXTRA_DEFAULT)
        : cstr((size_t)extra)
        , orcstrstream(*(cstr *)this,0)
        {}
    ocstrstream(const char * S, int extra = 0)
        : cstr(S,(size_t)extra)
        , orcstrstream(*(cstr *)this,0)
        {}
    char * str() { return orcstrstream::str(); }
    void pack() { orcstrstream::pack(); }
};

class rcstrstream : public strstream
{
    int _packed, _pcount, _packOnDestruct;
    cstr& cs_;
    size_t csStartLen;
  public:
    rcstrstream(cstr& cs, int packOnDestruct);
    char * str() { return (char *)cs_.str(); }
    strstreambuf * rdbuf() const
        { assert(0); return 0; }
    void reset();
    int pcount();
    void pack();
    ~rcstrstream();
};

class cstrstream : cstr, public rcstrstream
{
  public:
    cstrstream(int extra = CSTRSTRM_EXTRA_DEFAULT)
        : cstr((size_t)extra)
        , rcstrstream(*(cstr *)this,0)
        {}
    cstrstream(const char * S, int extra = 0)
        : cstr(S,(size_t)extra)
        , rcstrstream(*(cstr *)this,0)
        {}
    char * str() { return rcstrstream::str(); }
    void pack() { rcstrstream::pack(); }
};

#endif
